@@ -6,7 +6,7 @@ from scipy.stats import poisson, norm, chi2
 from scipy.linalg import solve_triangular
 from ..io.file import FileIOMixin
 from .format import ParameterFormatter, CostFunctionFormatter
-
+from ..util import log_determinant
 
 if six.PY2:
     from funcsigs import signature
@@ -17,6 +17,7 @@ else:
 __all__ = ["CostFunction",
            "CostFunction_Chi2",
            "CostFunction_NegLogLikelihood",
+           "CostFunction_GaussApproximation",
            "CostFunctionException"]
 
 
@@ -98,12 +99,16 @@ class CostFunction(FileIOMixin, object):
             self._arg_count += 2
         self._add_determinant_cost = add_determinant_cost
         if self._add_determinant_cost:
-            self._arg_names += ["total_cov_mat_log_determinant"]
+            if self.pointwise:
+                self._arg_names += ["total_error_squared_log_sum"]
+            else:
+                self._arg_names += ["total_cov_mat_log_determinant"]
             self._arg_count += 1
 
         self._needs_errors = True
         self._is_chi2 = False
         self._saturated = False
+        self._kafe2go_identifier = None
         super(CostFunction, self).__init__()
 
     @classmethod
@@ -175,6 +180,21 @@ class CostFunction(FileIOMixin, object):
         """Whether the determinant cost is being added automatically to the cost function value."""
         return self._add_determinant_cost
 
+    @property
+    def kafe2go_identifier(self):
+        """Short string representation (if any) of this cost function when dumping to file."""
+        return self._kafe2go_identifier
+
+    @property
+    def pointwise(self):
+        """True if cost function result does not depend on covariances."""
+        return False
+
+    @property
+    def pointwise_version(self):
+        """Optimized version of cost function that uses pointwise errors, can be None."""
+        return None
+
     def goodness_of_fit(self, *args):
         """How well the model agrees with the data."""
         try:
@@ -280,6 +300,7 @@ class CostFunction_Chi2(CostFunction):
         self._needs_errors = errors_to_use is not None
         self._is_chi2 = True
         self._saturated = True
+        self._kafe2go_identifier = self.name
 
     def _chi2(self, data, model, cov_mat_cholesky=None, err=None):
         data = np.asarray(data)
@@ -298,42 +319,41 @@ class CostFunction_Chi2(CostFunction):
                 _x = solve_triangular(cov_mat_cholesky, _res, lower=True)
             except ValueError:
                 return np.inf
-            _cost = np.sum(np.square(_x))
-            return _cost
+            return np.inner(_x, _x)
 
         if self._fail_on_no_matrix:
             raise CostFunctionException("Covariance matrix is singular!")
 
         # otherwise, if an array of point-wise errors is given, use that
         if err is not None:
-            err = np.asarray(err)
             if np.any(err == 0.0):
                 if self._fail_on_no_errors:
                     raise CostFunctionException("'err' must not contain any zero values!")
+                if self.needs_errors:
+                    # There are other warnings that notify the user about singular cov mat, etc.
+                    warnings.warn("Setting all data errors to 1 as a fallback.")
             else:
                 _res = _res / err
 
-        if self.needs_errors:
-            # There are other warnings that notify the user about singular cov mat, etc.
-            warnings.warn("Setting all data errors to 1 as a fallback.")
         # return sum of squared residuals
-        _cost = np.sum(_res ** 2)
+        _cost = np.inner(_res, _res)
         if np.isnan(_cost):
             _cost = np.inf
         return _cost
 
     def chi2_no_errors(self, data, model):
-        r"""A least-squares cost function calculated from `y` data and model values,
+        r"""A least-squares cost function calculated from (`y`) data and model values,
         without considering uncertainties:
 
         .. math::
-            C = \chi^2({\bf d}, {\bf m}) = ({\bf d} - {\bf m})\cdot({\bf d} - {\bf m})
+            C = \chi^2({\bf d}, {\bf m}) = \sum_k (d_k - m_k)^2
                 +
-                C({\bf p})
+                C_{\rm con}({\bf p}).
 
         In the above, :math:`{\bf d}` are the measurements,
         :math:`{\bf m}` are the model predictions,
-        and :math:`C({\bf p})` is the additional cost resulting from any constrained parameters.
+        and :math:`C_{\rm con}({\bf p})` is the additional cost resulting from any constrained
+        parameters.
 
         :param data: measurement data :math:`{\bf d}`
         :param model: model predictions :math:`{\bf m}`
@@ -343,8 +363,8 @@ class CostFunction_Chi2(CostFunction):
         return self._chi2(data=data, model=model)
 
     def chi2_covariance(self, data, model, total_cov_mat_cholesky):
-        r"""A least-squares cost function calculated from `y` data and model values,
-        considering the covariance matrix of the `y` measurements.
+        r"""A least-squares cost function calculated from (`y`) data and model values,
+        considering the covariance matrix of the (`y`) measurements.
         The cost function value can be calculated as follows:
 
         .. math::
@@ -353,7 +373,7 @@ class CostFunction_Chi2(CostFunction):
                 +
                 C_{\rm con}({\bf p})
                 +
-                C_{\rm det}({\bf V})
+                C_{\rm det}({\bf V}).
 
         In the above, :math:`{\bf d}` are the measurements,
         :math:`{\bf m}` are the model predictions,
@@ -366,25 +386,30 @@ class CostFunction_Chi2(CostFunction):
         :param data: measurement data :math:`{\bf d}`
         :param model: model predictions :math:`{\bf m}`
         :param total_cov_mat_cholesky: Cholesky decomposition  of the total covariance matrix
-            :math:`{\bf L}` with :math:`{\bf L}^T {\bf L} = {\bf V}`
+            :math:`{\bf L}` with :math:`{\bf L}^\top {\bf L} = {\bf V}`
 
         :return: cost function value
         """
         return self._chi2(data=data, model=model, cov_mat_cholesky=total_cov_mat_cholesky)
 
     def chi2_pointwise_errors(self, data, model, total_error):
-        r"""A least-squares cost function calculated from `y` data and model values,
+        r"""A least-squares cost function calculated from (`y`) data and model values,
         considering pointwise (uncorrelated) uncertainties for each data point:
 
         .. math::
             C = \chi^2({\bf d}, {\bf m}, {\bf \sigma}) = \sum_k \frac{d_k - m_k}{\sigma_k}
                 +
-                C({\bf p})
+                C_{\rm con}({\bf p})
+                +
+                C_{\rm det}({\bf \sigma}).
 
         In the above, :math:`{\bf d}` are the measurements,
         :math:`{\bf m}` are the model predictions,
         :math:`{\bf \sigma}` are the pointwise total uncertainties,
-        and :math:`C({\bf p})` is the additional cost resulting from any constrained parameters.
+        :math:`C_{\rm con}({\bf p})` is the additional cost resulting from any constrained
+        parameters,
+        and :math:`C_{\rm det}({\bf \sigma}) = \ln \prod_k \sigma_k^2` is the additional cost
+        to compensate for non-constant errors.
 
         :param data: measurement data :math:`{\bf d}`
         :param model: model predictions :math:`{\bf m}`
@@ -394,6 +419,23 @@ class CostFunction_Chi2(CostFunction):
         """
         return self._chi2(data=data, model=model, err=total_error)
 
+    @property
+    def pointwise(self):
+        return self._cost_function_handle != self.chi2_covariance
+
+    @property
+    def pointwise_version(self):
+        if self._cost_function_handle == self.chi2_covariance:
+            return type(self)(
+                errors_to_use="pointwise",
+                fallback_on_singular=not self._fail_on_no_matrix,
+                add_constraint_cost=self._add_constraint_cost,
+                add_determinant_cost=self._add_determinant_cost
+            )
+        else:
+            return None
+
+
 class CostFunction_NegLogLikelihood(CostFunction):
     def __init__(self, data_point_distribution='poisson', ratio=False):
         r"""
@@ -456,6 +498,7 @@ class CostFunction_NegLogLikelihood(CostFunction):
         self._formatter.description = _cost_function_description
         self._needs_errors = _nll_func in [self.nll_gaussian, self.nllr_gaussian]
         self._saturated = ratio
+        self._kafe2go_identifier = self.name
 
     @staticmethod
     def nll_gaussian(data, model, total_error):
@@ -466,18 +509,19 @@ class CostFunction_NegLogLikelihood(CostFunction):
         .. math::
             C = -2 \ln \mathcal{L}({\bf d}, {\bf m}, {\bf \sigma})
               = -2 \ln \prod_j \mathcal{L}_{\rm Gaussian} (x=d_j, \mu=m_j, \sigma=\sigma_j)
-                + C({\bf p})
+                + C_{\rm con}({\bf p}).
 
         .. math::
             \rightarrow C = -2 \ln \prod_j \frac{1}{\sqrt{2{\sigma_j}^2\pi}}
                             \exp{\left(-\frac{ (d_j-m_j)^2 }{ {\sigma_j}^2}\right)}
                             +
-                            C({\bf p})
+                            C_{\rm con}({\bf p}).
 
         In the above, :math:`{\bf d}` are the measurements,
         :math:`{\bf m}` are the model predictions,
         :math:`{\bf \sigma}` are the pointwise total uncertainties,
-        and :math:`C({\bf p})` is the additional cost resulting from any constrained parameters.
+        and :math:`C_{\rm con}({\bf p})` is the additional cost resulting from any constrained
+        parameters.
 
         :param data: measurement data :math:`{\bf d}`
         :param model: model predictions :math:`{\bf m}`
@@ -502,16 +546,17 @@ class CostFunction_NegLogLikelihood(CostFunction):
             C = -2 \ln \mathcal{L}({\bf d}, {\bf m})
               = -2 \ln \prod_j \mathcal{L}_{\rm Poisson} (k=d_j, \lambda=m_j)
                 +
-                C({\bf p})
+                C_{\rm con}({\bf p}).
 
         .. math::
             \rightarrow C = -2 \ln \prod_j \frac{{m_j}^{d_j} \exp(-m_j)}{d_j!}
                             +
-                            C({\bf p})
+                            C_{\rm con}({\bf p}).
 
         In the above, :math:`{\bf d}` are the measurements,
         :math:`{\bf m}` are the model predictions,
-        and :math:`C({\bf p})` is the additional cost resulting from any constrained parameters.
+        and :math:`C_{\rm con}({\bf p})` is the additional cost resulting from any constrained
+        parameters.
 
         :param data: measurement data :math:`{\bf d}`
         :param model: model predictions :math:`{\bf m}`
@@ -557,12 +602,167 @@ class CostFunction_NegLogLikelihood(CostFunction):
         return 0
 
 
+class CostFunction_GaussApproximation(CostFunction):
+    def __init__(
+            self, errors_to_use='covariance', add_constraint_cost=True, add_determinant_cost=True):
+        """
+        Base class for built-in Gaussian approximation of the Poisson negative log-likelihood cost
+        function.
+
+        :param errors_to_use: Which errors to use when calculating :math:`\\chi^2`.
+            Either ``'covariance'``, ``'pointwise'``.
+        :type errors_to_use: str
+        :param bool add_constraint_cost: If :py:obj:`True`, automatically add the cost for kafe2
+            constraints.
+        :param bool add_determinant_cost: If :py:obj:`True`, automatically increase the cost
+            function value by the logarithm of the determinant of the covariance matrix to reduce
+            bias.
+        """
+
+        _cost_function_description = "Gaussian approximation of Poisson NLL"
+        if errors_to_use.lower() == 'covariance':
+            _cost_function = self.gaussian_approximation_covariance
+            _arg_names = [self._DATA_NAME, self._MODEL_NAME, self._COV_MAT_CHOLESKY_NAME]
+            _cost_function_description += " (with covariance matrix)"
+        elif errors_to_use.lower() == 'pointwise':
+            _cost_function = self.gaussian_approximation_pointwise_errors
+            _arg_names = [self._DATA_NAME, self._MODEL_NAME, self._ERROR_NAME]
+            _cost_function_description += " (with pointwise errors)"
+        else:
+            raise ValueError(
+                "Unknown value '%s' for 'errors_to_use': must be one of "
+                "('covariance', 'pointwise')")
+
+        super().__init__(
+            cost_function=_cost_function,
+            arg_names=_arg_names,
+            add_constraint_cost=add_constraint_cost,
+            add_determinant_cost=False
+        )
+        self._add_determinant_cost_ga = add_determinant_cost
+
+        self._formatter.latex_name = r"-2\ln\tilde{\mathcal{L}}"
+        self._formatter.latex_name_saturated = r"-2\ln\tilde{\mathcal{L}}_{\rm R}"
+        self._formatter.description = _cost_function_description
+        self._needs_errors = False
+        self._is_chi2 = False
+        self._saturated = True
+        self._kafe2go_identifier = self.name
+
+    def gaussian_approximation_covariance(self, data, model, total_cov_mat_cholesky):
+        r"""A least-squares cost function calculated from (`y`) data and model values,
+        considering the covariance matrix of the (`y`) measurements.
+        The cost function value can be calculated as follows:
+
+        .. math::
+            C = \chi^2({\bf d}, {\bf m}, {\bf V})
+            = ({\bf d} - {\bf m})^{\top}\,{\tilde{\bf V}^{-1}}\,({\bf d} - {\bf m})
+                +
+                C_{\rm con}({\bf p})
+                +
+                C_{\rm det}(\tilde{\bf V}); \quad
+            \tilde{{\bf V}}_{ij} = {\bf V}_{ij} + \delta_{ij} {\bf m}_i.
+
+        In the above, :math:`{\bf d}` are the measurements,
+        :math:`{\bf m}` are the model predictions,
+        :math:`{{\bf V}^{-1}}` is the inverse of the total covariance matrix,
+        :math:`C_{\rm con}({\bf p})` is the additional cost resulting from any constrained
+        parameters,
+        and :math:`C_{\rm det}(\tilde{\bf V}) = \ln \det(\tilde{\bf V})` is the additional cost to
+        compensate for a non-constant covariance matrix.
+
+        :param data: measurement data :math:`{\bf d}`
+        :param model: model predictions :math:`{\bf m}`
+        :param total_cov_mat_cholesky: Cholesky decomposition  of the total covariance matrix
+            :math:`{\bf L}` with :math:`{\bf L}^\top {\bf L} = {\bf V}`
+
+        :return: cost function value
+        """
+        _cholesky = total_cov_mat_cholesky + np.diag(np.sqrt(model))
+        _residuals = model - data
+        try:
+            _x = solve_triangular(_cholesky, _residuals, lower=True)
+        except ValueError:
+            return np.inf
+        _cost = np.sum(np.square(_x))
+        if self._add_determinant_cost_ga:
+            _cost += log_determinant(_cholesky)
+        return _cost
+
+    def gaussian_approximation_pointwise_errors(self, data, model, total_error):
+        r"""A least-squares cost function calculated from data and model values,
+        considering pointwise (uncorrelated) uncertainties for each data point:
+
+        .. math::
+            C = \chi^2({\bf d}, {\bf m}, {\bf \sigma})
+                = \sum_k \left( \frac{d_k - m_k}{\sigma_k + \sqrt{m_k}} \right)^2
+                +
+                C_{\rm con}({\bf p})
+                +
+                C_{\rm det}({\bf \sigma}).
+
+        In the above, :math:`{\bf d}` are the measurements,
+        :math:`{\bf m}` are the model predictions,
+        :math:`{\bf \sigma}` are the pointwise total uncertainties,
+        :math:`C({\bf p})` is the additional cost resulting from any constrained parameters,
+        and :math:`C_{\rm det}({\bf \sigma}) = \ln \prod_k m_k + \sigma_k^2` is the additional cost
+        to compensate for non-constant errors.
+
+        :param data: measurement data :math:`{\bf d}`
+        :param model: model predictions :math:`{\bf m}`
+        :param total_error: total error vector :math:`{\bf \sigma}`
+
+        :return: cost function value
+        """
+        _residuals = model - data
+        if np.all(_residuals == 0):
+            return 0
+        _variances = model + total_error ** 2
+        _cost = np.sum(np.square(_residuals) / _variances)
+        if self._add_determinant_cost_ga:
+            _cost += np.sum(np.log(_variances))
+        if np.isnan(_cost):
+            _cost = np.inf
+        return _cost
+
+    @property
+    def pointwise(self):
+        return self._cost_function_handle == self.gaussian_approximation_pointwise_errors
+
+    @property
+    def pointwise_version(self):
+        if self._cost_function_handle == self.gaussian_approximation_covariance:
+            return type(self)(
+                errors_to_use="pointwise",
+                add_constraint_cost=self._add_constraint_cost,
+                add_determinant_cost=self._add_determinant_cost
+            )
+        else:
+            return None
+
+    def goodness_of_fit(self, *args):
+        _original_add_determinant_cost_pp = self._add_determinant_cost_ga
+        self._add_determinant_cost_ga = False
+        _gof = super().goodness_of_fit(*args)
+        self._add_determinant_cost_ga = _original_add_determinant_cost_pp
+        return _gof
+
+    def get_uncertainty_gaussian_approximation(self, data):
+        return np.sqrt(data)
+
+
 STRING_TO_COST_FUNCTION = {
     'chi2': (CostFunction_Chi2, {}),
     'chi_2': (CostFunction_Chi2, {}),
     'chisquared': (CostFunction_Chi2, {}),
     'chi_squared': (CostFunction_Chi2, {}),
+    'chi2_no_errors': (CostFunction_Chi2, {"errors_to_use": None}),
+    'chi2_pointwise': (CostFunction_Chi2, {"errors_to_use": "pointwise"}),
+    'chi2_pointwise_errors': (CostFunction_Chi2, {"errors_to_use": "pointwise"}),
+    'chi2_covariance': (CostFunction_Chi2, {"errors_to_use": "covariance"}),
     'nll': (CostFunction_NegLogLikelihood, {"ratio": False}),
+    'poisson': (CostFunction_NegLogLikelihood,
+                {"data_point_distribution": "poisson", "ratio": False}),
     'nll-poisson': (CostFunction_NegLogLikelihood,
                     {"data_point_distribution": "poisson", "ratio": False}),
     'nll_poisson': (CostFunction_NegLogLikelihood,
@@ -592,4 +792,12 @@ STRING_TO_COST_FUNCTION = {
                     {"data_point_distribution": "gaussian", "ratio": True}),
     'negloglikelihoodratio': (CostFunction_NegLogLikelihood, {"ratio": True}),
     'neg_log_likelihood_ratio': (CostFunction_NegLogLikelihood, {"ratio": True}),
+    'gauss-approximation': (CostFunction_GaussApproximation, {}),
+    'gauss_approximation': (CostFunction_GaussApproximation, {}),
+    'gauss_approximation_covariance': (
+        CostFunction_GaussApproximation, {"errors_to_use": "covariance"}),
+    'gauss_approximation_pointwise': (
+        CostFunction_GaussApproximation, {"errors_to_use": "pointwise"}),
+    'gauss_approximation_pointwise_errors': (
+        CostFunction_GaussApproximation, {"errors_to_use": "pointwise"}),
 }
