@@ -1,11 +1,14 @@
 import abc
 import sys
+import os
 import warnings
 from collections import OrderedDict
 from functools import partial
+from typing import Optional
 
 import numpy as np
 import six
+import yaml
 
 from .container import DataContainerBase, DataContainerException
 from ..io.file import FileIOMixin
@@ -16,7 +19,8 @@ from ...core.constraint import GaussianMatrixParameterConstraint, GaussianSimple
 from ...core.error import CovMat
 from ...tools import print_dict_as_table
 from .._base.cost import CostFunction, STRING_TO_COST_FUNCTION
-from ..util import invert_matrix, add_in_quadrature, cholesky_decomposition, log_determinant
+from ..util import invert_matrix, is_diagonal, cholesky_decomposition, log_determinant, \
+    log_determinant_pointwise, to_python_floats, to_numpy_arrays
 
 __all__ = ["FitBase", "FitException"]
 
@@ -37,7 +41,7 @@ class FitBase(FileIOMixin, object):
     MODEL_FUNCTION_TYPE = None
     PLOT_ADAPTER_TYPE = None
     EXCEPTION_TYPE = FitException
-    RESERVED_NODE_NAMES = None
+    RESERVED_NODE_NAMES = set()
     _BASIC_ERROR_NAMES = {}
     _STRING_TO_COST_FUNCTION = STRING_TO_COST_FUNCTION
     _AXES = (None,)  # axes for which to for example create data nexus nodes
@@ -109,6 +113,8 @@ class FitBase(FileIOMixin, object):
             # TODO: validate user-defined cost function? how?
         else:
             self._cost_function = None
+        self._cost_function_pointwise = None if self._cost_function is None \
+            else self._cost_function.pointwise_version
 
         # initialize the Nexus
         self._init_nexus()
@@ -127,7 +133,8 @@ class FitBase(FileIOMixin, object):
         obj = obj if obj is not None else self
         _node = self._nexus.add_function(
             partial(getattr(obj.__class__, prop).fget, obj),
-            func_name=name or prop
+            func_name=name or prop,
+            par_names=[]
         )
         if depends_on is not None:
             self._nexus.add_dependency(name=_node.name, depends_on=depends_on)
@@ -157,7 +164,7 @@ class FitBase(FileIOMixin, object):
 
         self._nexus.add(Array(_parameter_nodes, name="parameter_values"))
         self._nexus.add_function(lambda: self.parameter_constraints,
-                                 func_name='parameter_constraints')
+                                 func_name='parameter_constraints', par_names=[])
 
         # -- errors
 
@@ -192,6 +199,11 @@ class FitBase(FileIOMixin, object):
                                          par_names=[_mat_name])
                 self._nexus.add_function(log_determinant, func_name=_mat_name + "_log_determinant",
                                          par_names=[_mat_name + "_cholesky"])
+                self._nexus.add_function(
+                    log_determinant_pointwise,
+                    func_name=_error_name + "_squared_log_sum",
+                    par_names=[_error_name]
+                )
 
         if self._model_function is not None:
             # add the original function name as an alias to 'model'
@@ -209,6 +221,12 @@ class FitBase(FileIOMixin, object):
             )
 
             _cost_alias = self._nexus.add_alias('cost', alias_for=self._cost_function.name)
+        if self._cost_function_pointwise is not None:
+            self._nexus.add_function(
+                self._cost_function_pointwise,
+                par_names=self._cost_function_pointwise.arg_names,
+                func_name=self._cost_function_pointwise.name
+            )
 
     def _initialize_fitter(self):
         self._fitter = NexusFitter(nexus=self._nexus,
@@ -494,7 +512,11 @@ class FitBase(FileIOMixin, object):
     @property
     def total_cor_mat(self):
         """the total correlation matrix"""
-        return CovMat(self.total_cov_mat).cor_mat
+        _cov_mat = self.total_cov_mat
+        if _cov_mat is None:
+            return None
+        else:
+            return CovMat(self.total_cov_mat).cor_mat
 
     @property
     def model_function(self):
@@ -612,7 +634,7 @@ class FitBase(FileIOMixin, object):
 
         :rtype: bool
         """
-        return self._param_model.has_errors
+        return self._param_model is not None and self._param_model.has_errors
 
     @property
     def has_data_errors(self):
@@ -620,7 +642,7 @@ class FitBase(FileIOMixin, object):
 
         :rtype: bool
         """
-        return self._data_container.has_errors
+        return self._data_container is not None and self._data_container.has_errors
 
     @property
     def has_errors(self):
@@ -628,7 +650,7 @@ class FitBase(FileIOMixin, object):
 
         :rtype: bool
         """
-        return True if self.has_data_errors or self.has_model_errors else False
+        return self.has_data_errors or self.has_model_errors
 
     @property
     def model_count(self):
@@ -661,8 +683,12 @@ class FitBase(FileIOMixin, object):
 
     @property
     def goodness_of_fit(self):
-        return self._cost_function.goodness_of_fit(
-            *[self._nexus.get(_node_name).value for _node_name in self._cost_function.arg_names])
+        if self._cost_function_pointwise is not None and is_diagonal(self.total_cov_mat):
+            _cost_function = self._cost_function_pointwise
+        else:
+            _cost_function = self._cost_function
+        return _cost_function.goodness_of_fit(
+            *[self._nexus.get(_node_name).value for _node_name in _cost_function.arg_names])
 
     @property
     def dynamic_error_algorithm(self):
@@ -973,6 +999,13 @@ class FitBase(FileIOMixin, object):
         if self._cost_function.needs_errors and not self.has_errors:
             warnings.warn("Cost function expects errors but no errors were specified.")
 
+        if self._cost_function_pointwise is not None:
+            if is_diagonal(self.total_cov_mat):
+                _cost_target = self._cost_function_pointwise.name
+            else:
+                _cost_target = self._cost_function.name
+            self._fitter.parameter_to_minimize = _cost_target
+
         # Give relative model errors data as reference for initial fit:
         self._set_data_as_model_ref()
 
@@ -1145,3 +1178,41 @@ class FitBase(FileIOMixin, object):
         if calculate_asymmetric_errors:
             _ = self.asymmetric_parameter_errors  # force calculation of asymmetric errors if not calculated yet
         super(FitBase, self).to_file(filename=filename, file_format=file_format)
+
+    def save_state(
+            self, filename: str, file_format: Optional[str] = None,
+            calculate_asymmetric_errors: bool = False):
+        if file_format is None:
+            _, _extension = os.path.splitext(filename)
+            if len(_extension) >= 2:
+                file_format = _extension[1:]
+        if file_format is None:
+            file_format = "yml"
+        file_format = file_format.lower()
+
+        if calculate_asymmetric_errors:
+            _ = self.asymmetric_parameter_errors
+
+        if file_format in ("yaml", "yml"):
+            with open(filename, "w", encoding="utf8") as _f:
+                yaml.safe_dump(to_python_floats(self.get_result_dict()), _f)
+        else:
+            raise ValueError(f"Unknown file format: {file_format}. Available: yaml")
+
+    def load_state(self, filename: str, file_format: Optional[str] = None):
+        if file_format is None:
+            _, _extension = os.path.splitext(filename)
+            if len(_extension) >= 2:
+                file_format = _extension[1:]
+        if file_format is None:
+            file_format = "yml"
+        file_format = file_format.lower()
+
+        if file_format in ("yaml", "yml"):
+            with open(filename, "r", encoding="utf8") as _f:
+                self._loaded_result_dict = to_numpy_arrays(yaml.safe_load(_f))
+        else:
+            raise ValueError(f"Unknown file format: {file_format}. Available: yaml")
+        _new_par_vals = self._loaded_result_dict.pop("parameter_values", None)
+        if _new_par_vals is not None:
+            self._fitter.set_all_fit_parameter_values(_new_par_vals)
