@@ -1,23 +1,25 @@
-import numpy as np
-import six
 import textwrap
 import tokenize
 
-from .._base import DReprError
-from .._yaml_base import YamlWriterMixin, YamlReaderMixin
-from ..container import DataContainerYamlWriter, DataContainerYamlReader
-from ..format import ModelFunctionFormatterYamlWriter, ModelFunctionFormatterYamlReader
+import numpy as np
+import six
+
 from ._base import ModelFunctionDReprBase, ParametricModelDReprBase
 from .. import _AVAILABLE_REPRESENTATIONS
+from .._base import DReprError
+from .._yaml_base import YamlReaderException, YamlReaderMixin, YamlWriterException, YamlWriterMixin
+from ..error import common_error_tools
+from ..format import ModelFunctionFormatterYamlReader, ModelFunctionFormatterYamlWriter
 from ..._base import ModelFunctionBase
-from ... import HistModelFunction, IndexedModelFunction
-from ... import HistParametricModel, IndexedParametricModel, UnbinnedParametricModel, XYParametricModel
-from .._yaml_base import YamlWriterException, YamlReaderException
+from ....fit import HistModelFunction, HistParametricModel, IndexedModelFunction, \
+    IndexedParametricModel, UnbinnedParametricModel, XYParametricModel
 from ....fit.util import function_library
 
-__all__ = ['ModelFunctionYamlWriter', 'ModelFunctionYamlReader', 'ParametricModelYamlWriter', 'ParametricModelYamlReader']
+__all__ = ['ModelFunctionYamlWriter', 'ModelFunctionYamlReader', 'ParametricModelYamlWriter',
+           'ParametricModelYamlReader']
 KNOWN_MODEL_FUNCTIONS = (ModelFunctionBase, HistModelFunction, IndexedModelFunction)
-KNOWN_PARAMETRIC_MODELS = (XYParametricModel, HistParametricModel, IndexedParametricModel, UnbinnedParametricModel)
+KNOWN_PARAMETRIC_MODELS = (XYParametricModel, HistParametricModel, IndexedParametricModel,
+                           UnbinnedParametricModel)
 
 
 def _parse_function(input_string):
@@ -76,7 +78,7 @@ class ModelFunctionYamlWriter(YamlWriterMixin, ModelFunctionDReprBase):
         super(ModelFunctionYamlWriter, self).__init__(
             output_io_handle=output_io_handle,
             model_function=model_function)
-    
+
     @classmethod
     def _make_representation(cls, model_function):
         _yaml_doc = dict()
@@ -88,18 +90,19 @@ class ModelFunctionYamlWriter(YamlWriterMixin, ModelFunctionDReprBase):
             raise DReprError("Model function type unknown or not supported: %s" % _class)
         if _type != 'base':  # 'base' is default, no need to store type
             _yaml_doc['type'] = _type  # store all other custom types
-        _yaml_doc['model_function_formatter'] = ModelFunctionFormatterYamlWriter._make_representation(model_function.formatter)
+        _yaml_doc['model_function_formatter'] =\
+            ModelFunctionFormatterYamlWriter._make_representation(model_function.formatter)
 
         _yaml_doc['python_code'] = _process_function_code_for_dump(model_function.source_code)
-        
+
         return _yaml_doc
 
 
 class ModelFunctionYamlReader(YamlReaderMixin, ModelFunctionDReprBase):
-
-    FORBIDDEN_TOKENS = ['compile', 'eval', 'exec', 'execfile', 'file', 'global', 'globals', 'import', '__import__', 
+    FORBIDDEN_TOKENS = ['compile', 'eval', 'exec', 'execfile', 'file', 'global', 'globals',
+                        'import', '__import__',
                         'input', ' locals', 'nonlocal', 'open', 'reload', 'self', 'super']
-    
+
     def __init__(self, input_io_handle):
         super(ModelFunctionYamlReader, self).__init__(
             input_io_handle=input_io_handle,
@@ -107,11 +110,12 @@ class ModelFunctionYamlReader(YamlReaderMixin, ModelFunctionDReprBase):
 
     @classmethod
     def _get_subspace_override_dict(cls, model_function_class):
-        _override_dict = {'arg_formatters': 'model_function_formatter',
-                          'model_function_name': 'model_function_formatter',
-                          'latex_model_function_name': 'model_function_formatter',
+        _override_dict = {'name': 'model_function_formatter',
+                          'latex_name': 'model_function_formatter',
+                          'arg_formatters': 'model_function_formatter',
                           'expression_string': 'model_function_formatter',
-                          'latex_expression_string': 'model_function_formatter'}
+                          'latex_expression_string': 'model_function_formatter',
+                          'signature': 'model_function_formatter'}
         if model_function_class is IndexedModelFunction:
             _override_dict['index_name'] = 'model_function_formatter'
             _override_dict['latex_index_name'] = 'model_function_formatter'
@@ -122,11 +126,11 @@ class ModelFunctionYamlReader(YamlReaderMixin, ModelFunctionDReprBase):
         if issubclass(model_function_class, ModelFunctionBase):
             return ['python_code']
         raise YamlReaderException("Unknown model function class: %s" % model_function_class)
-        
+
     @classmethod
     def _process_string(cls, string_representation, default_type):
         return dict(type=default_type, python_code=string_representation)
-    
+
     @classmethod
     def _convert_yaml_doc_to_object(cls, yaml_doc):
         """Converts the given yaml dictionary to a :py:obj:``ModelFunctionBase`` or derived object.
@@ -147,24 +151,26 @@ class ModelFunctionYamlReader(YamlReaderMixin, ModelFunctionDReprBase):
             _parsed_function = _parse_function(_raw_string)
             _model_function_object = _class(_parsed_function)
             _model_function_object._source_code = _raw_string
-        
-        #construct model function formatter if specified
-        # TODO: only overwrite given arguments, use defaults if one arg is not specified
+
+        # construct model function formatter if specified
         _model_function_formatter_yaml = yaml_doc.pop('model_function_formatter', None)
         if _model_function_formatter_yaml:
+            _name = _model_function_object.name
+            _signature = _model_function_object.signature.parameters
             _model_function_object._formatter = ModelFunctionFormatterYamlReader._make_object(
-                _model_function_formatter_yaml)
-        
+                _model_function_formatter_yaml, default_type=_model_function_type, name=_name,
+                signature=_signature)
+
         return _model_function_object, yaml_doc
 
 
 class ParametricModelYamlWriter(YamlWriterMixin, ParametricModelDReprBase):
+    DUMPER = common_error_tools.MatrixYamlDumper
 
     def __init__(self, parametric_model, output_io_handle):
-        super(ParametricModelYamlWriter, self).__init__(
-            output_io_handle=output_io_handle,
-            parametric_model=parametric_model)
-    
+        super(ParametricModelYamlWriter, self).__init__(output_io_handle=output_io_handle,
+                                                        parametric_model=parametric_model)
+
     @classmethod
     def _make_representation(cls, parametric_model):
         """Create a dictionary representation of a parametric model.
@@ -181,7 +187,7 @@ class ParametricModelYamlWriter(YamlWriterMixin, ParametricModelDReprBase):
         if _type is None:
             raise DReprError("Parametric model type unknown or not supported: %s" % _class)
         _yaml_doc['type'] = _type
-        
+
         # -- write representation for model types
         if _class is HistParametricModel:
             _yaml_doc['n_bins'] = int(parametric_model.n_bins)
@@ -218,13 +224,14 @@ class ParametricModelYamlWriter(YamlWriterMixin, ParametricModelDReprBase):
 
         # -- write error representation for all container types
         if parametric_model.has_errors:
-            DataContainerYamlWriter._write_errors_to_yaml(parametric_model, _yaml_doc)
+            _yaml_doc = common_error_tools.write_errors_to_yaml(container=parametric_model,
+                                                                yaml_doc=_yaml_doc)
 
         return _yaml_doc
 
 
 class ParametricModelYamlReader(YamlReaderMixin, ParametricModelDReprBase):
-    
+
     def __init__(self, input_io_handle):
         super(ParametricModelYamlReader, self).__init__(
             input_io_handle=input_io_handle,
@@ -255,7 +262,7 @@ class ParametricModelYamlReader(YamlReaderMixin, ParametricModelDReprBase):
         else:
             raise YamlReaderException("Unknown parametric model type")
         return _override_dict
-    
+
     @classmethod
     def _get_required_keywords(cls, yaml_doc, parametric_model_class):
         if parametric_model_class is HistParametricModel:
@@ -271,14 +278,34 @@ class ParametricModelYamlReader(YamlReaderMixin, ParametricModelDReprBase):
     def _get_ignored_if_none_keywords(cls):
         return ["model_density_func_antiderivative"]
 
+    @classmethod
+    def _modify_yaml_doc(cls, yaml_doc, kafe_object_class, dataset=None, **kwargs):
+        if dataset:
+            if kafe_object_class is HistParametricModel:
+                if 'bin_edges' not in yaml_doc and ('n_bins' not in yaml_doc or 'bin_range' not
+                                                    in yaml_doc):
+                    yaml_doc['bin_edges'] = dataset.bin_edges
+            elif kafe_object_class is IndexedParametricModel:
+                if 'shape_like' not in yaml_doc:
+                    yaml_doc['shape_like'] = dataset.data
+            elif kafe_object_class is UnbinnedParametricModel:
+                if 'data' not in yaml_doc:
+                    yaml_doc['data'] = dataset.data
+            elif kafe_object_class is XYParametricModel:
+                if 'x_data' not in yaml_doc:
+                    yaml_doc['x_data'] = dataset.x
+        super(ParametricModelYamlReader, cls)._modify_yaml_doc(yaml_doc, kafe_object_class,
+                                                               **kwargs)
+        return yaml_doc
+
     @classmethod
     def _convert_yaml_doc_to_object(cls, yaml_doc):
         # -- determine model function class from type
         _parametric_model_type = yaml_doc.pop('type')
         _class = cls._OBJECT_TYPE_NAME_TO_CLASS.get(_parametric_model_type, None)
 
-        #_kwarg_list = kwargs that directly correspond to unchanging scalar yaml entries
-        _kwarg_list = []         
+        # _kwarg_list = kwargs that directly correspond to unchanging scalar yaml entries
+        _kwarg_list = []
         _constructor_kwargs = {}
         _hist_model_bin_evaluation_source = None  # Set if an antiderivative function is read in.
         if _class is HistParametricModel:
@@ -290,7 +317,7 @@ class ParametricModelYamlReader(YamlReaderMixin, ParametricModelDReprBase):
                 _constructor_kwargs['bin_edges'] = _bin_edges
                 _constructor_kwargs['n_bins'] = len(_bin_edges) - 1
                 _constructor_kwargs['bin_range'] = [_bin_edges[0], _bin_edges[-1]]
-                
+
             elif 'n_bins' in yaml_doc and 'bin_range' in yaml_doc:
                 _kwarg_list.append('n_bins')
                 _kwarg_list.append('bin_range')
@@ -318,17 +345,20 @@ class ParametricModelYamlReader(YamlReaderMixin, ParametricModelDReprBase):
         if _class is HistParametricModel:
             _model_func_entry = yaml_doc.pop('model_density_function', None)
             if _model_func_entry:
-                _model_func = ModelFunctionYamlReader._make_object(_model_func_entry, default_type=_parametric_model_type)
+                _model_func = ModelFunctionYamlReader._make_object(
+                    _model_func_entry, default_type=_parametric_model_type)
                 _constructor_kwargs['model_density_func'] = _model_func
         elif _class is UnbinnedParametricModel:
             _model_func_entry = yaml_doc.pop('model_function', None)
             if _model_func_entry:
-                _model_func = ModelFunctionYamlReader._make_object(_model_func_entry, default_type=_parametric_model_type)
+                _model_func = ModelFunctionYamlReader._make_object(
+                    _model_func_entry, default_type=_parametric_model_type)
                 _constructor_kwargs['model_density_function'] = _model_func
         elif _class in (IndexedParametricModel, XYParametricModel):
             _model_func_entry = yaml_doc.pop('model_function', None)
             if _model_func_entry:
-                _model_func = ModelFunctionYamlReader._make_object(_model_func_entry, default_type=_parametric_model_type)
+                _model_func = ModelFunctionYamlReader._make_object(
+                    _model_func_entry, default_type=_parametric_model_type)
                 _constructor_kwargs['model_func'] = _model_func
         else:
             raise YamlReaderException('Unknown model type: %s' % _parametric_model_type)
@@ -350,49 +380,11 @@ class ParametricModelYamlReader(YamlReaderMixin, ParametricModelDReprBase):
 
         # add the label for all types
         _parametric_model_object.label = yaml_doc.pop('model_label', None)
-        
+
         # -- process error sources
-        if _class is XYParametricModel:
-            _xerrs = yaml_doc.pop('x_errors', [])
-            _yerrs = yaml_doc.pop('y_errors', [])
-            _errs = _xerrs + _yerrs
-            _axes = [0] * len(_xerrs) + [1] * len(_yerrs)  # 0 for 'x', 1 for 'y'
-        else:
-            _errs = yaml_doc.pop('errors', [])
-            _axes = [None] * len(_errs)
+        _parametric_model_object, yaml_doc = common_error_tools.process_error_sources(
+            container_obj=_parametric_model_object, yaml_doc=yaml_doc)
 
-        # add error sources, if any
-        for _err, _axis in zip(_errs, _axes):
-            _add_kwargs = dict()
-            # translate and check that all required keys are present
-            try:
-                _err_type = _err['type']
-
-                _add_kwargs['name'] = _err['name']
-
-                if _err_type == 'simple':
-                    _add_kwargs['err_val']= _err['error_value']
-                    _add_kwargs['correlation']= _err['correlation_coefficient']
-                elif _err_type == 'matrix':
-                    _add_kwargs['err_matrix'] = _err['matrix']
-                    _add_kwargs['matrix_type'] = _err['matrix_type']
-                    _add_kwargs['err_val'] = _err.get('error_value', None)  # only mandatory for cor mats; check done later
-                else:
-                    raise DReprError("Unknown error type '{}'. "
-                                     "Valid: {}".format(_err_type, ('simple', 'matrix')))
-
-                _add_kwargs['relative'] = _err['relative']
-
-                # if needed, specify the axis (only for 'xy' containers)
-                if _axis is not None:
-                    _add_kwargs['axis'] = _axis
-            except KeyError as e:
-                # KeyErrors mean the YAML is incomplete -> raise
-                raise DReprError("Missing required key '%s' for error specification" % e.args[0])
-
-            # add error to parametric model
-            DataContainerYamlReader._add_error_to_container(_err_type, _parametric_model_object, **_add_kwargs)
-        
         return _parametric_model_object, yaml_doc
 
 
