@@ -6,12 +6,15 @@ r"""This submodule provides utility functions for other modules.
 """
 
 import warnings
+from collections import OrderedDict
+from typing import List, Optional
+
 import numpy as np
 
 from . import function_library
 
-__all__ = ['string_join_if', 'add_in_quadrature', 'invert_matrix', 'cholesky_decomposition', 'log_determinant',
-           'collect']
+__all__ = ['string_join_if', 'add_in_quadrature', 'invert_matrix', 'cholesky_decomposition',
+           'log_determinant', 'log_determinant_pointwise', 'collect', 'is_diagonal']
 
 # -- general utility functions
 
@@ -61,6 +64,42 @@ def log_determinant(cholesky_mat):
         return 2.0 * np.sum(np.log(np.diag(cholesky_mat)))
 
 
+def log_determinant_pointwise(pointwise_error):
+    """
+    Calculate the logarithm of the determinant of a matrix from its Cholesky decomposition.
+    """
+    return 2.0 * np.sum(np.log(pointwise_error))
+
+
 def collect(*args):
     '''collect arguments into array'''
     return np.asarray(args)
+
+
+def is_diagonal(matrix):
+    """Return True if only diagonal matrix elements are non-zero."""
+    return np.all(matrix - np.diag(np.diagonal(matrix)) == 0)
+
+def to_python_floats(yaml_dict: Optional[dict]):
+    if yaml_dict is None:
+        return None
+    _new_dict = {}
+    for _key, _value in yaml_dict.items():
+        if isinstance(_value, float):
+            _value = float(_value)  # Casts NumPy scalars to Python float.
+        elif isinstance(_value, np.ndarray):
+            _value = _value.tolist()
+        elif isinstance(_value, OrderedDict):
+            _value = [float(_v) for _v in _value.values()]
+        _new_dict[_key] = _value
+    return _new_dict
+
+def to_numpy_arrays(yaml_dict: Optional[dict]):
+    if yaml_dict is None:
+        return None
+    _new_dict = {}
+    for _key, _value in yaml_dict.items():
+        if isinstance(_value, list):
+            _value = np.array(_value)
+        _new_dict[_key] = _value
+    return _new_dict
