@@ -95,91 +95,90 @@ class TestXYFitBasicInterface(AbstractTestFit, unittest.TestCase):
 
         # helper dict with all reference property values
         self._ref_prop_dict = dict(
-            parameter_values=self._ref_initial_pars,
-            poi_values=self._ref_initial_pars,
-
-            x_data=self._ref_x,
-            x_model=self._ref_x,
-            y_data=self._ref_y_data,
-            y_model=self._ref_initial_y_model,
-            data=self._ref_xy_data,
-            model=self._ref_initial_xy_model,
-            poi_names=('a', 'b', 'c'),
             did_fit=False,
             model_count=1,
+
+            parameter_values=self._ref_initial_pars,
+            parameter_names=('a', 'b', 'c'),
             cost_function_value=self._ref_initial_cost,
 
-            x_data_cov_mat=self._ref_matrix_eye * 0,
-            y_data_cov_mat=self._ref_matrix_eye,
-            x_data_uncor_cov_mat=self._ref_matrix_eye * 0,
-            y_data_uncor_cov_mat=self._ref_matrix_eye,
-            x_data_cov_mat_inverse=None,
-            y_data_cov_mat_inverse=self._ref_matrix_eye,
-            #x_data_uncor_cov_mat_inverse=None,  # TODO: fix
-            y_data_uncor_cov_mat_inverse=self._ref_matrix_eye,
-            #x_data_cor_mat=self._ref_matrix_eye,  # TODO: fix
-            y_data_cor_mat=self._ref_matrix_eye,
+            x_data=self._ref_x,
+            y_data=self._ref_y_data,
+            data=self._ref_xy_data,
             x_data_error=self._ref_error * 0,
             y_data_error=self._ref_error,
+            data_error=self._ref_error,
+            x_data_cov_mat=self._ref_matrix_eye * 0,
+            y_data_cov_mat=self._ref_matrix_eye,
+            data_cov_mat=self._ref_matrix_eye,
+            x_data_cov_mat_inverse=None,
+            y_data_cov_mat_inverse=self._ref_matrix_eye,
+            data_cov_mat_inverse=self._ref_matrix_eye,
+            x_data_cor_mat=self._ref_matrix_eye * np.nan,
+            y_data_cor_mat=self._ref_matrix_eye,
+            data_cor_mat=self._ref_matrix_eye,
 
-            x_model_cov_mat=self._ref_matrix_eye * 0,
-            y_model_cov_mat=self._ref_matrix_eye * 0,
-            x_model_uncor_cov_mat=self._ref_matrix_eye * 0,
-            y_model_uncor_cov_mat=self._ref_matrix_eye * 0,
-            x_model_cov_mat_inverse=None,
-            y_model_cov_mat_inverse=None,
-            #x_model_uncor_cov_mat_inverse=None,  # TODO: fix
-            #y_model_uncor_cov_mat_inverse=None,  # TODO: fix
-            #x_model_cor_mat=self._ref_matrix_eye,  # TODO: fix
-            #y_model_cor_mat=self._ref_matrix_eye,  # TODO: fix
+            x_model=self._ref_x,
+            y_model=self._ref_initial_y_model,
+            model=self._ref_initial_xy_model,
             x_model_error=self._ref_error * 0,
             y_model_error=self._ref_error * 0,
+            model_error=self._ref_error * 0,
+            x_model_cov_mat=self._ref_matrix_eye * 0,
+            y_model_cov_mat=self._ref_matrix_eye * 0,
+            model_cov_mat=self._ref_matrix_eye * 0,
+            x_model_cov_mat_inverse=None,
+            y_model_cov_mat_inverse=None,
+            model_cov_mat_inverse=None,
+            x_model_cor_mat=self._ref_matrix_eye * np.nan,
+            y_model_cor_mat=self._ref_matrix_eye * np.nan,
+            model_cor_mat=self._ref_matrix_eye * np.nan,
 
-            x_total_cov_mat=self._ref_matrix_eye * 0,
-            y_total_cov_mat=self._ref_matrix_eye,
-            x_total_uncor_cov_mat=self._ref_matrix_eye * 0,
-            y_total_uncor_cov_mat=self._ref_matrix_eye,
-            x_total_cov_mat_inverse=None,
-            y_total_cov_mat_inverse=self._ref_matrix_eye,
-            #x_total_uncor_cov_mat_inverse=None,  # TODO: fix
-            y_total_uncor_cov_mat_inverse=self._ref_matrix_eye,
-            #x_total_cor_mat=self._ref_matrix_eye,  # TODO: add
-            #y_total_cor_mat=self._ref_matrix_eye,  # TODO: add
             x_total_error=self._ref_error * 0,
             y_total_error=self._ref_error,
-
-            projected_xy_total_error=self._ref_error,
-            projected_xy_total_cov_mat=self._ref_matrix_eye,
-
-            _y_data_nuisance_cor_design_mat=np.zeros((0, self._n_points)),
+            total_error=self._ref_error,
+            x_total_cov_mat=self._ref_matrix_eye * 0,
+            y_total_cov_mat=self._ref_matrix_eye,
+            total_cov_mat=self._ref_matrix_eye,
+            x_total_cov_mat_inverse=None,
+            y_total_cov_mat_inverse=self._ref_matrix_eye,
+            total_cov_mat_inverse=self._ref_matrix_eye,
+            x_total_cor_mat=self._ref_matrix_eye * np.nan,
+            y_total_cor_mat=self._ref_matrix_eye,
+            total_cor_mat=self._ref_matrix_eye,
         )
 
-    def _get_fit(self, model_function=None, cost_function=None, error=None):
+    def _get_fit(
+            self, model_function=None, cost_function=None, errors=None,
+            dynamic_error_algorithm=None):
         '''convenience'''
         model_function = model_function or simple_xy_model
         # TODO: fix default
         cost_function = cost_function or XYCostFunction_Chi2(
             axes_to_use='xy', errors_to_use='covariance')
-        error = error or 1.0
+        errors = errors or [dict(axis='y', err_val=1.0)]
+        dynamic_error_algorithm = dynamic_error_algorithm or "nonlinear"
 
         _fit = XYFit(
             xy_data=self._ref_xy_data,
             model_function=model_function,
             cost_function=cost_function,
-            minimizer=self.MINIMIZER
+            minimizer=self.MINIMIZER,
+            dynamic_error_algorithm=dynamic_error_algorithm
         )
-        _fit.add_error(axis='y', err_val=error)
+        for _err in errors:
+            _fit.add_error(**_err)
 
         return _fit
 
     def _get_test_fits(self):
         return {
-            'default': \
-                self._get_fit(),
-            'explicit': \
-                self._get_fit(cost_function=simple_chi2),
-            'explicit_model': \
-                self._get_fit(cost_function=simple_chi2_explicit_model_name),
+            'default': self._get_fit(),
+            'explicit': self._get_fit(cost_function=simple_chi2),
+            'explicit_model': self._get_fit(cost_function=simple_chi2_explicit_model_name),
+            'relative_errors_data': self._get_fit(errors=[
+                dict(axis="y", err_val=1.0/self._ref_y_data, relative=True, reference="data")
+            ])
         }
 
     def test_initial_state(self):
@@ -191,7 +190,6 @@ class TestXYFitBasicInterface(AbstractTestFit, unittest.TestCase):
         self.run_test_for_all_fits(
             dict(self._ref_prop_dict,
                 parameter_values=self._nominal_fit_result_pars,
-                poi_values=self._nominal_fit_result_pars,
                 y_model=self._nominal_fit_result_y_model,
                 model=self._nominal_fit_result_xy_model,
                 did_fit=True,
@@ -205,14 +203,13 @@ class TestXYFitBasicInterface(AbstractTestFit, unittest.TestCase):
         self.run_test_for_all_fits(
             dict(self._ref_prop_dict,
                 parameter_values=self._nominal_fit_result_pars,
-                poi_values=self._nominal_fit_result_pars,
                 x_data=self._ref_x,
                 x_model=self._ref_x,
                 y_data=self._ref_y_data,
                 y_model=self._nominal_fit_result_y_model,
                 data=self._ref_xy_data,
                 model=self._nominal_fit_result_xy_model,
-                poi_names=('a', 'b', 'c'),
+                parameter_names=('a', 'b', 'c'),
                 did_fit=False,
                 model_count=1,
                 cost_function_value=self._nominal_fit_result_cost
@@ -231,33 +228,6 @@ class TestXYFitBasicInterface(AbstractTestFit, unittest.TestCase):
         with self.assertRaises(NexusFitterException):
             self._get_fit().set_all_parameter_values((1,2,3,4,5))
 
-    def test_set_poi_values(self):
-        self.run_test_for_all_fits(
-            dict(self._ref_prop_dict,
-                parameter_values=self._nominal_fit_result_pars,
-                poi_values=self._nominal_fit_result_pars,
-                x_data=self._ref_x,
-                x_model=self._ref_x,
-                y_data=self._ref_y_data,
-                y_model=self._nominal_fit_result_y_model,
-                data=self._ref_xy_data,
-                model=self._nominal_fit_result_xy_model,
-                poi_names=('a', 'b', 'c'),
-                did_fit=False,
-                model_count=1,
-                cost_function_value=self._nominal_fit_result_cost
-            ),
-            # set parameters to their final values by hand
-            call_before_fit=lambda f: f.set_poi_values(
-                self._nominal_fit_result_pars),
-        )
-
-    def test_set_poi_values_wrong_number_raise(self):
-        with self.assertRaises(XYFitException):
-            self._get_fit().set_poi_values(param_values=(1,))
-        with self.assertRaises(XYFitException):
-            self._get_fit().set_poi_values(param_values=(1,2,3,4,5))
-
     def test_parameter_defaults(self):
         def dummy_model(x, a, b, c):
             return x
@@ -268,7 +238,6 @@ class TestXYFitBasicInterface(AbstractTestFit, unittest.TestCase):
             _fit,
             dict(
                 parameter_values=np.array([1, 1, 1]),
-                poi_values=np.array([1, 1, 1]),
             )
         )
 
@@ -282,7 +251,6 @@ class TestXYFitBasicInterface(AbstractTestFit, unittest.TestCase):
             _fit,
             dict(
                 parameter_values=np.array([1, 1, 3.3]),
-                poi_values=np.array([1, 1, 3.3]),
             )
         )
 
@@ -326,13 +294,6 @@ class TestXYFitBasicInterface(AbstractTestFit, unittest.TestCase):
             self._ref_initial_y_model
         ))
 
-    def test_calculate_nuisance_parameters(self):
-        _fit = self._get_fit()
-        self.assertTrue(np.allclose(
-            _fit.calculate_nuisance_parameters(),
-            []
-        ))
-
     def test_update_data(self):
         _fit = self._get_fit()
 
@@ -358,7 +319,6 @@ class TestXYFitBasicInterface(AbstractTestFit, unittest.TestCase):
                 x_data=self._ref_x,
                 y_data=self._ref_y_data * 2,
                 parameter_values=_new_estimates,
-                poi_values=_new_estimates,
             ),
             rtol=1e-2
         )
@@ -393,7 +353,6 @@ class TestXYFitBasicInterface(AbstractTestFit, unittest.TestCase):
                 x_data=self._ref_x[:-1],
                 y_data=self._ref_y_data[:-1] * 2,
                 parameter_values=_new_estimates,
-                poi_values=_new_estimates,
             ),
             rtol=1e-2
         )
@@ -492,6 +451,45 @@ class TestXYFitBasicInterface(AbstractTestFit, unittest.TestCase):
         _fit.report(output_stream=_buffer)
         self.assertNotEqual(_buffer.getvalue(), "")
 
+    def test_unknown_cost_function(self):
+        with self.assertRaises(ValueError):
+            self._get_fit(cost_function="ABC123")
+
+    def test_model_as_model_func_name(self):
+        def model(x, a, b):
+            return a * x + b
+        self._get_fit(model_function=model)
+
+    def test_data_and_cost_incompatible(self):
+        with self.assertRaises(XYFitException):
+            self._get_fit(cost_function="nll")
+
+    def test_relative_model_error_nonlinear(self):
+        _fit_data_err = self._get_fit(errors=[
+            dict(axis="y", err_val=1.0, relative=False, reference="data"),
+            dict(axis="y", err_val=0.1, relative=True, reference="data")
+        ])
+        _fit_model_err = self._get_fit(errors=[
+            dict(axis="y", err_val=1.0, relative=False, reference="data"),
+            dict(axis="y", err_val=0.1, relative=True, reference="model")
+        ], dynamic_error_algorithm="nonlinear")
+        _fit_data_err.do_fit()
+        _fit_model_err.do_fit()
+        self._assert_fit_results_equal(_fit_data_err, _fit_model_err, rtol=6e-2)
+
+    def test_relative_model_error_iterative(self):
+        _fit_data_err = self._get_fit(errors=[
+            dict(axis="y", err_val=1.0, relative=False, reference="data"),
+            dict(axis="y", err_val=0.1, relative=True, reference="data")
+        ])
+        _fit_model_err = self._get_fit(errors=[
+            dict(axis="y", err_val=1.0, relative=False, reference="data"),
+            dict(axis="y", err_val=0.1, relative=True, reference="model")
+        ], dynamic_error_algorithm="iterative")
+        _fit_data_err.do_fit()
+        _fit_model_err.do_fit()
+        self._assert_fit_results_equal(_fit_data_err, _fit_model_err, rtol=1e-2)
+
 
 class TestXYFitWithSimpleYErrors(AbstractTestFit, unittest.TestCase):
 
@@ -547,7 +545,6 @@ class TestXYFitWithSimpleYErrors(AbstractTestFit, unittest.TestCase):
         self.run_test_for_all_fits(
             dict(self._ref_prop_dict,
                 parameter_values=self._nominal_fit_result_pars,
-                poi_values=self._nominal_fit_result_pars,
                 y_model=self._nominal_fit_result_y_model,
                 model=self._nominal_fit_result_xy_model,
                 did_fit=True,
@@ -689,7 +686,6 @@ class TestXYFitWithMatrixErrors(AbstractTestFit, unittest.TestCase):
         self.run_test_for_all_fits(
             dict(self._ref_prop_dict,
                 parameter_values=self._nominal_fit_result_pars,
-                poi_values=self._nominal_fit_result_pars,
                 y_model=self._nominal_fit_result_y_model,
                 model=self._nominal_fit_result_xy_model,
                 did_fit=True,
@@ -764,76 +760,91 @@ class TestXYFitWithXYErrors(AbstractTestFit, unittest.TestCase):
     MINIMIZER = 'scipy'
 
     def setUp(self):
+        """
+        This was changed because the results were suspicious when cov mat determinant was added
+        to cost.
+        """
         self._n_points = 16
+        self._x_error = 0.1
+        self._y_error = 0.1
         self._default_cost_function = XYCostFunction_Chi2(
             axes_to_use='xy', errors_to_use='covariance')
 
         # "jitter" for data smearing
-        self._y_jitter = np.array([ 0.2991,  1.558 , -0.714 ,  0.825 , -1.157 ,  0.462 ,  0.103 ,
-                                    1.167 ,  0.783 , -0.688 , -1.019 ,  0.14  , -0.11  , -0.87  ,
-                                    1.81  ,  0.35  ])
+        np.random.seed(0)
+        self._x_jitter = np.random.normal(scale=self._x_error, size=self._n_points)
+        self._y_jitter = np.random.normal(scale=self._y_error, size=self._n_points)
 
         assert len(self._y_jitter) == self._n_points
 
         # reference initial values
-        self._ref_initial_pars = np.array([1.0, 0.0])
-        self._ref_x = np.arange(self._n_points)
-        self._ref_initial_y_model = line_xy_model(self._ref_x, *self._ref_initial_pars)
-        self._ref_initial_xy_model = np.array([self._ref_x, self._ref_initial_y_model])
+        self._ref_initial_pars = np.array([1.0, 0.5])
+        self._ref_x_data = np.arange(self._n_points) + self._x_jitter
+        self._ref_initial_y_model = line_xy_model(self._ref_x_data, *self._ref_initial_pars)
+        self._ref_initial_xy_model = np.array([self._ref_x_data, self._ref_initial_y_model])
 
         # reference matrices/errors
-        self._ref_y_error_vector = np.ones(self._n_points) * 0.1
+        self._ref_y_error_vector = np.ones(self._n_points) * self._y_error
         self._ref_y_error_matrix = np.diag(self._ref_y_error_vector ** 2)
-        self._ref_x_error_vector = np.ones(self._n_points)
+        self._ref_x_error_vector = np.ones(self._n_points) * self._x_error
         self._ref_x_error_matrix = np.diag(self._ref_x_error_vector ** 2)
 
         def line_xy_model_derivative(x, a=1.0, b=0.0):
             return a
 
-        _fp = line_xy_model_derivative(self._ref_x, *self._ref_initial_pars)
+        _fp = line_xy_model_derivative(self._ref_x_data, *self._ref_initial_pars)
         self._ref_projected_xy_matrix = (
-            self._ref_y_error_matrix  +
+            self._ref_y_error_matrix +
             self._ref_x_error_matrix * np.outer(_fp, _fp)
         )
         self._ref_projected_xy_errors = np.diag(np.sqrt(self._ref_projected_xy_matrix))
+        self._ref_projected_xy_log_determinant = np.log(np.linalg.det(
+            self._ref_projected_xy_matrix))
 
         # fit data
-        self._ref_y_data = self._ref_initial_y_model + self._y_jitter
-        self._ref_xy_data = np.array([self._ref_x, self._ref_y_data])
+        self._ref_y_data = line_xy_model(self._ref_x_data, *self._ref_initial_pars) + self._y_jitter
+        self._ref_xy_data = np.array([self._ref_x_data, self._ref_y_data])
 
         # pre-fit cost value
         self._ref_initial_cost = self._default_cost_function(
             self._ref_y_data,
             self._ref_initial_y_model,
-            np.linalg.inv(self._ref_projected_xy_matrix),
+            np.linalg.cholesky(self._ref_projected_xy_matrix),
             self._ref_initial_pars,
             [],
+            self._ref_projected_xy_log_determinant
         )
 
         # reference fit result values
-        self._nominal_fit_result_pars = np.array([1.02590618, -0.00967721])
+        self._nominal_fit_result_pars = np.array([1.00441642, 0.48645824])
 
-        self._nominal_fit_result_y_model = line_xy_model(self._ref_x, *self._nominal_fit_result_pars)
-        self._nominal_fit_result_xy_model = np.array([self._ref_x, self._nominal_fit_result_y_model])
+        self._nominal_fit_result_y_model = line_xy_model(
+            self._ref_x_data, *self._nominal_fit_result_pars)
+        self._nominal_fit_result_xy_model = np.array(
+            [self._ref_x_data, self._nominal_fit_result_y_model])
 
-        self._nominal_fit_result_projected_xy_matrix = 1.0624835 * np.eye(self._n_points)
-        self._nominal_fit_result_projected_xy_errors = np.diag(np.sqrt(self._nominal_fit_result_projected_xy_matrix))
+        self._nominal_fit_result_projected_xy_matrix = self._ref_y_error_matrix\
+            + self._nominal_fit_result_pars[0] * self._ref_x_error_matrix
+        self._nominal_fit_result_projected_xy_errors = np.diag(np.sqrt(
+            self._nominal_fit_result_projected_xy_matrix))
+        self._nominal_fit_result_projected_xy_log_determinant = np.log(np.linalg.det(
+            self._nominal_fit_result_projected_xy_matrix))
 
         self._nominal_fit_result_cost = self._default_cost_function(
             self._ref_y_data,
             self._nominal_fit_result_y_model,
-            np.linalg.inv(self._nominal_fit_result_projected_xy_matrix),
+            np.linalg.cholesky(self._nominal_fit_result_projected_xy_matrix),
             self._nominal_fit_result_pars,
             [],
+            self._nominal_fit_result_projected_xy_log_determinant
         )
 
         # helper dict with all reference property values
         self._ref_prop_dict = dict(
             parameter_values=self._ref_initial_pars,
-            poi_values=self._ref_initial_pars,
 
-            x_data=self._ref_x,
-            x_model=self._ref_x,
+            x_data=self._ref_x_data,
+            x_model=self._ref_x_data,
             y_data=self._ref_y_data,
             y_model=self._ref_initial_y_model,
             data=self._ref_xy_data,
@@ -842,12 +853,8 @@ class TestXYFitWithXYErrors(AbstractTestFit, unittest.TestCase):
 
             x_data_cov_mat=self._ref_x_error_matrix,
             y_data_cov_mat=self._ref_y_error_matrix,
-            x_data_uncor_cov_mat=self._ref_x_error_matrix,
-            y_data_uncor_cov_mat=self._ref_y_error_matrix,
             x_data_cov_mat_inverse=np.linalg.inv(self._ref_x_error_matrix),
             y_data_cov_mat_inverse=np.linalg.inv(self._ref_y_error_matrix),
-            x_data_uncor_cov_mat_inverse=np.linalg.inv(self._ref_x_error_matrix),
-            y_data_uncor_cov_mat_inverse=np.linalg.inv(self._ref_y_error_matrix),
             x_data_cor_mat=np.eye(self._n_points),
             y_data_cor_mat=np.eye(self._n_points),
             x_data_error=self._ref_x_error_vector,
@@ -855,25 +862,29 @@ class TestXYFitWithXYErrors(AbstractTestFit, unittest.TestCase):
 
             cost_function_value=self._ref_initial_cost,
 
-            projected_xy_total_cov_mat=self._ref_projected_xy_matrix,
-            projected_xy_total_error=self._ref_projected_xy_errors,
+            total_cov_mat=self._ref_projected_xy_matrix,
+            total_error=self._ref_projected_xy_errors,
         )
 
-    def _get_fit(self, errors=None):
+    def _get_fit(self, errors=None, constraints=None, dynamic_error_algorithm=None):
         '''convenience'''
 
         errors = errors or [dict(axis='y', err_val=1.0)]
+        constraints = constraints or []
+        dynamic_error_algorithm = dynamic_error_algorithm or "nonlinear"
 
         _fit = XYFit(
             xy_data=self._ref_xy_data,
             model_function=line_xy_model,
             cost_function=self._default_cost_function,
             minimizer=self.MINIMIZER,
-            #x_error_algorithm='nonlinear',  # TODO: test other algorithms
+            dynamic_error_algorithm=dynamic_error_algorithm
         )
 
         for _err in errors:
             _fit.add_error(**_err)
+        for _constraint in constraints:
+            _fit.add_parameter_constraint(**_constraint)
 
         _fit.set_all_parameter_values(self._ref_initial_pars)
 
@@ -898,14 +909,13 @@ class TestXYFitWithXYErrors(AbstractTestFit, unittest.TestCase):
         self.run_test_for_all_fits(
             dict(self._ref_prop_dict,
                 parameter_values=self._nominal_fit_result_pars,
-                poi_values=self._nominal_fit_result_pars,
                 y_model=self._nominal_fit_result_y_model,
                 model=self._nominal_fit_result_xy_model,
                 did_fit=True,
                 cost_function_value=self._nominal_fit_result_cost,
 
-                projected_xy_total_cov_mat=self._nominal_fit_result_projected_xy_matrix,
-                projected_xy_total_error=self._nominal_fit_result_projected_xy_errors,
+                total_cov_mat=self._nominal_fit_result_projected_xy_matrix,
+                total_error=self._nominal_fit_result_projected_xy_errors,
 
             ),
             call_before_fit=lambda f: f.do_fit(),
@@ -927,3 +937,55 @@ class TestXYFitWithXYErrors(AbstractTestFit, unittest.TestCase):
         with self.assertRaises(TypeError):
             # string representations of numerics should also fail
             self._get_fit().limit_parameter("a", "0.3", "14")
+
+    def test_iterative_linear_and_relative_model_error(self):
+        _constraints = [
+            dict(name="a", value=1.0, uncertainty=0.1),
+            dict(name="b", value=0.3, uncertainty=0.1)
+        ]
+        _errors_rel_data = [
+            dict(axis="x", err_val=0.1, relative=False, reference="data"),
+            dict(axis="y", err_val=1.0, relative=False, reference="data"),
+            dict(axis="y", err_val=0.1, relative=True, reference="data")
+        ]
+        _errors_rel_model = [
+            dict(axis="x", err_val=0.1, relative=False, reference="data"),
+            dict(axis="y", err_val=1.0, relative=False, reference="data"),
+            dict(axis="y", err_val=0.1, relative=True, reference="model")
+        ]
+        _fit_rel_data_nonlinear = self._get_fit(
+            errors=_errors_rel_data, constraints=_constraints,
+            dynamic_error_algorithm="nonlinear")
+        _fit_rel_data_iterative = self._get_fit(
+            errors=_errors_rel_data, constraints=_constraints,
+            dynamic_error_algorithm="iterative")
+        _fit_rel_model_nonlinear = self._get_fit(
+            errors=_errors_rel_model, constraints=_constraints,
+            dynamic_error_algorithm="nonlinear")
+        _fit_rel_model_iterative = self._get_fit(
+            errors=_errors_rel_model, constraints=_constraints,
+            dynamic_error_algorithm="iterative")
+        _fit_rel_data_nonlinear.do_fit()
+        _fit_rel_data_iterative.do_fit()
+        _fit_rel_model_nonlinear.do_fit()
+        _fit_rel_model_iterative.do_fit()
+
+        print("============ rel data nonlinear vs rel data iterative ==========\n")
+        self._assert_fit_results_equal(
+            _fit_rel_data_nonlinear, _fit_rel_data_iterative, rtol=1.25e-2, atol=5e-5)
+        print("============ rel data nonlinear vs rel model nonlinear ==========\n")
+        self._assert_fit_results_equal(
+            _fit_rel_data_nonlinear, _fit_rel_model_nonlinear, rtol=1.25e-2, atol=5e-5)
+        print("============ rel data nonlinear vs rel model iterative ==========\n")
+        self._assert_fit_results_equal(
+            _fit_rel_data_nonlinear, _fit_rel_model_iterative, rtol=1.25e-2, atol=5e-5)
+        print("============ rel model nonlinear vs rel model iterative ==========\n")
+        self._assert_fit_results_equal(
+            _fit_rel_model_nonlinear, _fit_rel_model_iterative, rtol=1.25e-2, atol=5e-5)
+        print("============ rel data iterative vs rel model iterative ==========\n")
+        self._assert_fit_results_equal(
+            _fit_rel_data_iterative, _fit_rel_model_iterative, rtol=1.25e-2, atol=5e-5)
+        print("============ rel data iterative vs rel model nonlinear ==========\n")
+        self._assert_fit_results_equal(
+            _fit_rel_data_iterative, _fit_rel_model_nonlinear, rtol=1.25e-2, atol=5e-5)
+
